/*
 * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2008, 2009
 *	The President and Fellows of Harvard College.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _LAMEBUS_LTIMER_H_
#define _LAMEBUS_LTIMER_H_

/*
 * Hardware device data for LAMEbus timer device
 */
struct ltimer_softc {
	/* Initialized by config function */
	int lt_hardclock;        /* true if we should call hardclock() */
	int lt_timerclock;        /* true if we should call timerclock() */

	/* Initialized by lower-level attach routine */
	void *lt_bus;		/* bus we're on */
	uint32_t lt_buspos;	/* position (slot) on that bus */
	
};

/* Granularity of countdown timer (usec) */
/* Should be less than 1000000 */
#define LT_GRANULARITY   10000

/* Functions called by lower-level drivers */
void ltimer_irq(/*struct ltimer_softc*/ void *lt);  // interrupt handler

/* Functions called by higher-level devices */
void ltimer_beep(/*struct ltimer_softc*/ void *devdata);   // for beep device
void ltimer_gettime(/*struct ltimer_softc*/ void *devdata,
		    time_t *secs, uint32_t *nsecs);       // for rtclock

#endif /* _LAMEBUS_LTIMER_H_ */
